@@ -3,7 +3,11 @@ Local classifier per node approach.
 
 Numeric and string output labels are both handled.
 """
+
+import hashlib
+import pickle
 from copy import deepcopy
+from os.path import exists
 
 import networkx as nx
 import numpy as np
@@ -43,6 +47,7 @@ class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
         replace_classifiers: bool = True,
         n_jobs: int = 1,
         bert: bool = False,
+        tmp_dir: str = None,
     ):
         """
         Initialize a local classifier per node.
@@ -77,6 +82,9 @@ class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
             If :code:`Ray` is installed it is used, otherwise it defaults to :code:`Joblib`.
         bert : bool, default=False
             If True, skip scikit-learn's checks and sample_weight passing for BERT.
+        tmp_dir : str, default=None
+            Temporary directory to persist local classifiers that are trained. If the job needs to be restarted,
+            it will skip the pre-trained local classifier found in the temporary directory.
         """
         super().__init__(
             local_classifier=local_classifier,
@@ -86,6 +94,7 @@ class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
             n_jobs=n_jobs,
             classifier_abbreviation="LCPN",
             bert=bert,
+            tmp_dir=tmp_dir,
         )
         self.binary_policy = binary_policy
 
@@ -150,7 +159,7 @@ class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
 
         # Input validation
         if not self.bert:
-            X = check_array(X, accept_sparse="csr")
+            X = check_array(X, accept_sparse="csr", allow_nd=True, ensure_2d=False)
         else:
             X = np.array(X)
 
@@ -236,14 +245,28 @@ class LocalClassifierPerNode(BaseEstimator, HierarchicalClassifier):
     @staticmethod
     def _fit_classifier(self, node):
         classifier = self.hierarchy_.nodes[node]["classifier"]
+        if self.tmp_dir:
+            md5 = hashlib.md5(node.encode("utf-8")).hexdigest()
+            filename = f"{self.tmp_dir}/{md5}.sav"
+            if exists(filename):
+                (_, classifier) = pickle.load(open(filename, "rb"))
+                self.logger_.info(
+                    f"Loaded trained model for local classifier {node.split(self.separator_)[-1]} from file {filename}"
+                )
+                return classifier
+        self.logger_.info(f"Training local classifier {node}")
         X, y, sample_weight = self.binary_policy_.get_binary_examples(node)
         unique_y = np.unique(y)
         if len(unique_y) == 1 and self.replace_classifiers:
             classifier = ConstantClassifier()
         if not self.bert:
-            classifier.fit(X, y, sample_weight)
+            try:
+                classifier.fit(X, y, sample_weight)
+            except TypeError:
+                classifier.fit(X, y)
         else:
             classifier.fit(X, y)
+        self._save_tmp(node, classifier)
         return classifier
 
     def _clean_up(self):
