@@ -1,9 +1,13 @@
 """Init module for the library."""
-import os
-from ._version import get_versions
+
+from .LocalClassifierPerLevel import LocalClassifierPerLevel
 from .LocalClassifierPerNode import LocalClassifierPerNode
 from .LocalClassifierPerParentNode import LocalClassifierPerParentNode
-from .LocalClassifierPerLevel import LocalClassifierPerLevel
+from .MultiLabelLocalClassifierPerNode import MultiLabelLocalClassifierPerNode
+from .MultiLabelLocalClassifierPerParentNode import (
+    MultiLabelLocalClassifierPerParentNode,
+)
+from ._version import get_versions
 
 __version__ = get_versions()["version"]
 del get_versions
@@ -12,4 +16,6 @@ __all__ = [
     "LocalClassifierPerNode",
     "LocalClassifierPerParentNode",
     "LocalClassifierPerLevel",
+    "MultiLabelLocalClassifierPerNode",
+    "MultiLabelLocalClassifierPerParentNode",
 ]
